aboutsummaryrefslogtreecommitdiff
path: root/libbb/read_key.c
diff options
context:
space:
mode:
Diffstat (limited to 'libbb/read_key.c')
-rw-r--r--libbb/read_key.c21
1 files changed, 16 insertions, 5 deletions
diff --git a/libbb/read_key.c b/libbb/read_key.c
index 98b3131de..0faa12c97 100644
--- a/libbb/read_key.c
+++ b/libbb/read_key.c
@@ -9,7 +9,7 @@
*/
#include "libbb.h"
-int64_t FAST_FUNC read_key(int fd, char *buffer)
+int64_t FAST_FUNC read_key(int fd, char *buffer, int timeout)
{
struct pollfd pfd;
const char *seq;
@@ -90,14 +90,27 @@ int64_t FAST_FUNC read_key(int fd, char *buffer)
/* ESC [ Z - Shift-Tab */
};
+ pfd.fd = fd;
+ pfd.events = POLLIN;
+
buffer++; /* saved chars counter is in buffer[-1] now */
start_over:
errno = 0;
n = (unsigned char)buffer[-1];
if (n == 0) {
- /* If no data, block waiting for input.
- * It is tempting to read more than one byte here,
+ /* If no data, wait for input.
+ * If requested, wait TIMEOUT ms. TIMEOUT = -1 is useful
+ * if fd can be in non-blocking mode.
+ */
+ if (timeout >= -1) {
+ if (safe_poll(&pfd, 1, timeout) == 0) {
+ /* Timed out */
+ errno = EAGAIN;
+ return -1;
+ }
+ }
+ /* It is tempting to read more than one byte here,
* but it breaks pasting. Example: at shell prompt,
* user presses "c","a","t" and then pastes "\nline\n".
* When we were reading 3 bytes here, we were eating
@@ -121,8 +134,6 @@ int64_t FAST_FUNC read_key(int fd, char *buffer)
}
/* Loop through known ESC sequences */
- pfd.fd = fd;
- pfd.events = POLLIN;
seq = esccmds;
while (*seq != '\0') {
/* n - position in sequence we did not read yet */